Package: rotl
Title: Interface to the 'Open Tree of Life' API
Version: 3.1.1
Authors@R: 
    c(person(given = "Francois",
             family = "Michonneau",
             role = c("aut", "cre"),
             email = "francois.michonneau@gmail.com",
             comment = c(ORCID = "0000-0002-9092-966X")),
      person(given = "Joseph",
             family = "Brown",
             role = "aut",
             comment = c(ORCID = "0000-0002-3835-8062")),
      person(given = "David",
             family = "Winter",
             role = "aut",
             comment = c(ORCID = "0000-0002-6165-0029")),
      person(given = "Scott",
             family = "Chamberlain",
             role = "rev",
             comment = c(ORCID = "0000-0003-1444-9135")))
Description: An interface to the 'Open Tree of Life' API to retrieve
    phylogenetic trees, information about studies used to assemble the
    synthetic tree, and utilities to match taxonomic names to 'Open Tree
    identifiers'. The 'Open Tree of Life' aims at assembling a
    comprehensive phylogenetic tree for all named species.
License: BSD_2_clause + file LICENSE
URL: https://docs.ropensci.org/rotl/, https://github.com/ropensci/rotl
BugReports: https://github.com/ropensci/rotl/issues
Depends: R (>= 3.1.1)
Imports: ape, curl (>= 3.0.0), httr, jsonlite, rentrez, rlang, rncl (>=
        0.6.0)
Suggests: knitr (>= 1.12), MCMCglmm, phylobase, readxl, rmarkdown (>=
        0.7), RNeXML, testthat
VignetteBuilder: knitr
Encoding: UTF-8
X-schema.org-isPartOf: https://ropensci.org
X-schema.org-keywords: metadata, ropensci, phylogenetics,
        independant-contrasts, biodiversity
X-schema.org-relatedLink: https://codemeta.github.io/codemetar
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-01-15 17:36:05 UTC; francois
Author: Francois Michonneau [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9092-966X>),
  Joseph Brown [aut] (ORCID: <https://orcid.org/0000-0002-3835-8062>),
  David Winter [aut] (ORCID: <https://orcid.org/0000-0002-6165-0029>),
  Scott Chamberlain [rev] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>)
Maintainer: Francois Michonneau <francois.michonneau@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-15 18:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 19:18:55 UTC; unix
