# Generated by roxygen2: do not edit by hand

S3method("[",otl_ott_id)
S3method(candidate_for_synth,study_meta)
S3method(flags,match_names)
S3method(flags,taxon_info)
S3method(flags,taxon_mrca)
S3method(get_publication,study_meta)
S3method(get_study_year,study_meta)
S3method(get_tree_ids,study_meta)
S3method(inspect,match_names)
S3method(is_in_tree,default)
S3method(is_in_tree,otl_ott_id)
S3method(is_suppressed,match_names)
S3method(is_suppressed,taxon_info)
S3method(is_suppressed,taxon_mrca)
S3method(list_trees,matched_studies)
S3method(ott_id,match_names)
S3method(ott_id,taxon_info)
S3method(ott_id,taxon_mrca)
S3method(ott_id,tol_mrca)
S3method(ott_id,tol_node)
S3method(ott_id,tol_summary)
S3method(print,study_external_data)
S3method(print,study_ids)
S3method(print,study_meta)
S3method(print,tnrs_contexts)
S3method(print,tol_mrca)
S3method(print,tol_node)
S3method(print,tol_summary)
S3method(source_list,tol_mrca)
S3method(source_list,tol_node)
S3method(source_list,tol_summary)
S3method(synonyms,match_names)
S3method(synonyms,taxon_info)
S3method(tax_lineage,taxon_info)
S3method(tax_lineage,tol_node)
S3method(tax_name,match_names)
S3method(tax_name,taxon_info)
S3method(tax_name,taxon_mrca)
S3method(tax_name,tol_mrca)
S3method(tax_name,tol_node)
S3method(tax_name,tol_summary)
S3method(tax_rank,match_names)
S3method(tax_rank,taxon_info)
S3method(tax_rank,taxon_mrca)
S3method(tax_rank,tol_mrca)
S3method(tax_rank,tol_node)
S3method(tax_rank,tol_summary)
S3method(tax_sources,match_names)
S3method(tax_sources,taxon_info)
S3method(tax_sources,taxon_mrca)
S3method(tax_sources,tol_mrca)
S3method(tax_sources,tol_node)
S3method(tax_sources,tol_summary)
S3method(tol_lineage,tol_node)
S3method(unique_name,match_names)
S3method(unique_name,taxon_info)
S3method(unique_name,taxon_mrca)
S3method(unique_name,tol_mrca)
S3method(unique_name,tol_node)
S3method(unique_name,tol_summary)
S3method(update,match_names)
export(candidate_for_synth)
export(flags)
export(get_publication)
export(get_study)
export(get_study_meta)
export(get_study_subtree)
export(get_study_tree)
export(get_study_year)
export(get_tree_ids)
export(inspect)
export(is_in_tree)
export(is_suppressed)
export(list_trees)
export(ott_id)
export(source_list)
export(strip_ott_ids)
export(studies_find_studies)
export(studies_find_trees)
export(studies_properties)
export(study_external_IDs)
export(synonyms)
export(tax_lineage)
export(tax_name)
export(tax_rank)
export(tax_sources)
export(taxon_external_IDs)
export(taxonomy_about)
export(taxonomy_mrca)
export(taxonomy_subtree)
export(taxonomy_taxon_info)
export(tnrs_contexts)
export(tnrs_infer_context)
export(tnrs_match_names)
export(tol_about)
export(tol_induced_subtree)
export(tol_lineage)
export(tol_mrca)
export(tol_node_info)
export(tol_subtree)
export(unique_name)
import(ape)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,content)
importFrom(httr,parse_url)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,unbox)
importFrom(rentrez,entrez_link)
importFrom(rentrez,entrez_search)
importFrom(rlang,interrupt)
importFrom(rncl,read_newick_phylo)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stats,update)
